import pickle
import re2

class Re2ShieldDatabase:
    def __init__(self, re2_patterns, raw_patterns):
        self.re2_patterns = re2_patterns
        self.raw_patterns = raw_patterns

    def findall(self, text, callback):
        for id, pattern in self.re2_patterns.items():
            matches = pattern.finditer(text)
            for match in matches:
                callback(id, match.start(), match.end(), None, match.group())

    def count_patterns(self):
        return len(self.re2_patterns)

class Re2Shield:
    def __init__(self, version=None, pattern_counts=0):
        self.db = None
        self.version = version
        self.pattern_counts = pattern_counts

    def __str__(self):
        message = {
            "version": self.version,
            "pattern_counts": self.pattern_counts
        }
        return str(message)

    def compile(self, expressions, ids, overwrite=False):
        if len(expressions) != len(ids):
            raise ValueError("All parameters should have the same length")

        re2_patterns = {}
        raw_patterns = {}
        if not overwrite:
            re2_patterns = self.db.re2_patterns.copy()
            raw_patterns = self.db.raw_patterns.copy()

        for id, expr in zip(ids, expressions):
            if id in re2_patterns:
                raise ValueError(f"ID {id} already exists in the database. To overwrite, set overwrite=True.")
            re2_patterns[id] = re2.compile(expr)
            raw_patterns[id] = expr

        self.db = Re2ShieldDatabase(re2_patterns, raw_patterns)

    def dump(self, file_path):
        if self.db is not None:
            with open(file_path, 'wb') as f:
                pickle.dump((self.version, self.db.raw_patterns, self.db.count_patterns()), f)
        else:
            raise ValueError("No compiled database found. Please compile patterns first.")

    def scan(self, text, callback):
        if self.db is not None:
            self.db.findall(text, callback)
        else:
            raise ValueError("No compiled database found. Please compile patterns first.")

Database = Re2Shield

def load(file_path):
    with open(file_path, 'rb') as f:
        version, raw_patterns, pattern_counts = pickle.load(f)
        re2_patterns = {id: re2.compile(expr) for id, expr in raw_patterns.items()}
    re2_shield = Re2Shield(version=version, pattern_counts=pattern_counts)
    re2_shield.db = Re2ShieldDatabase(re2_patterns, raw_patterns)
    return re2_shield



import re2shield

if __name__ == "__main__":
    db = re2shield.Database(version='1.0.0')
    print(db)
    # Load patterns from file
    try:
        db = re2shield.load('patterns.db')
        print(db)  # Prints the number of patterns in the database
    except FileNotFoundError:
        # If pattern file doesn't exist, compile the patterns
        patterns = [
            (r'\b[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\.[A-Z|a-z]{2,}\b', 1),
            (r'\b\d{3}[-.\s]??\d{3}[-.\s]??\d{4}\b', 2),
            (r'\d+', 3)
        ]

        expressions, ids = zip(*patterns)
        db.compile(expressions=expressions, ids=ids, overwrite=False)
        print(db)  # Prints the number of patterns in the database and version
        db.dump('patterns.db')

    # Find patterns in text
    def match_handler(id, from_, to, flags, context):
        print(f"Match found for pattern {id} from {from_} to {to}: {context}")

    db.scan('test@ex12ample12.com', match_handler)
    print(db.version)
    print(db.pattern_counts)



from setuptools import setup, find_packages


with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='re2shield',
    version='0.1.9',
    url='https://github.com/Npc-coder/re2shield',  # Replace with your own GitHub project URL
    author='mkCha',
    author_email='dxsaq0@gmail.com',
    description='A Python library that provides a convenient interface for compiling and matching regular expression patterns using the re2 library.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),    
    install_requires=['google-re2', 'pickle5'],  # List your package's dependencies
    classifiers=[
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.8',
    ],
)
